@@ -139,7 +139,7 @@ corosync_node_name(uint64_t /*cmap_handle_t */ cmap_handle, uint32_t nodeid)
 void
 terminate_cs_connection(crm_cluster_t *cluster)
 {
-    crm_notice("Disconnecting from Corosync");
+    crm_info("Disconnecting from Corosync");
 
     cluster_disconnect_cpg(cluster);
 
@@ -151,6 +151,8 @@ terminate_cs_connection(crm_cluster_t *cluster)
     } else {
         crm_info("No Quorum connection");
     }
+
+    crm_notice("Disconnected from Corosync");
 }
 
 int ais_membership_timer = 0;
@@ -182,13 +184,19 @@ pcmk_quorum_notification(quorum_handle_t handle,
     static gboolean init_phase = TRUE;
 
     if (quorate != crm_have_quorum) {
-        crm_notice("Membership " U64T ": quorum %s (%lu)", ring_id,
-                   quorate ? "acquired" : "lost", (long unsigned int)view_list_entries);
+        if (quorate) {
+            crm_notice("Quorum acquired " CRM_XS " membership=" U64T " members=%lu",
+                       ring_id, (long unsigned int)view_list_entries);
+        } else {
+            crm_warn("Quorum lost " CRM_XS " membership=" U64T " members=%lu",
+                     ring_id, (long unsigned int)view_list_entries);
+        }
         crm_have_quorum = quorate;
 
     } else {
-        crm_info("Membership " U64T ": quorum %s (%lu)", ring_id,
-                 quorate ? "retained" : "still lost", (long unsigned int)view_list_entries);
+        crm_info("Quorum %s " CRM_XS " membership=" U64T " members=%lu",
+                 (quorate? "retained" : "still lost"), ring_id,
+                 (long unsigned int)view_list_entries);
     }
 
     if (view_list_entries == 0 && init_phase) {
@@ -268,7 +276,11 @@ cluster_connect_quorum(gboolean(*dispatch) (unsigned long long, gboolean),
         goto bail;
     }
 
-    crm_notice("Quorum %s", quorate ? "acquired" : "lost");
+    if (quorate) {
+        crm_notice("Quorum acquired");
+    } else {
+        crm_warn("Quorum lost");
+    }
     quorum_app_callback = dispatch;
     crm_have_quorum = quorate;
 
