@@ -214,11 +214,12 @@ stop_child(pcmk_child_t * child, int signal)
 
     errno = 0;
     if (kill(child->pid, signal) == 0) {
-        crm_notice("Stopping %s: Sent -%d to process %d", child->name, signal, child->pid);
+        crm_notice("Stopping %s "CRM_XS" sent signal %d to process %d",
+                   child->name, signal, child->pid);
 
     } else {
-        crm_perror(LOG_ERR, "Stopping %s: Could not send -%d to process %d failed",
-                   child->name, signal, child->pid);
+        crm_perror(LOG_ERR, "Could not stop %s (process %d) with signal %d",
+                   child->name, child->pid, signal);
     }
 
     return TRUE;
@@ -405,7 +406,8 @@ pcmk_shutdown_worker(gpointer user_data)
 
                 } else if (now >= next_log) {
                     next_log = now + 30;
-                    crm_notice("Still waiting for %s (pid=%d, seq=%d) to terminate...",
+                    crm_notice("Still waiting for %s to terminate "
+                               CRM_XS " pid=%d seq=%d",
                                child->name, child->pid, child->start_seq);
                 }
                 return TRUE;
@@ -699,11 +701,18 @@ check_active_before_startup_processes(gpointer user_data)
                 continue;
             } else if (start_seq != pcmk_children[lpc].start_seq) {
                 continue;
-            } else if (crm_pid_active(pcmk_children[lpc].pid, pcmk_children[lpc].name) != 1) {
-                crm_notice("Process %s terminated (pid=%d)",
-                           pcmk_children[lpc].name, pcmk_children[lpc].pid);
-                pcmk_process_exit(&(pcmk_children[lpc]));
-                continue;
+            } else {
+                const char *name = pcmk_children[lpc].name;
+                if (pcmk_children[lpc].flag == crm_proc_stonith_ng) {
+                    name = "stonithd";
+                }
+
+                if (crm_pid_active(pcmk_children[lpc].pid, name) != 1) {
+                    crm_notice("Process %s terminated (pid=%d)",
+                           name, pcmk_children[lpc].pid);
+                    pcmk_process_exit(&(pcmk_children[lpc]));
+                    continue;
+                }
             }
             /* at least one of the processes found at startup
              * is still going, so keep this recurring timer around */
@@ -821,7 +830,7 @@ mcp_cpg_deliver(cpg_handle_t handle,
     const char *task = crm_element_value(xml, F_CRM_TASK);
 
     crm_trace("Received CPG message (%s): %.200s",
-              (task? task : "process list"), msg);
+              (task? task : "process list"), (char*)msg);
 
     if (task == NULL) {
         if (nodeid == local_nodeid) {
@@ -1007,7 +1016,8 @@ main(int argc, char **argv)
         crm_exit(ENODATA);
     }
 
-    crm_notice("Starting Pacemaker %s (Build: %s): %s", PACEMAKER_VERSION, BUILD_VERSION, CRM_FEATURES);
+    crm_notice("Starting Pacemaker %s "CRM_XS" build=%s features:%s",
+               PACEMAKER_VERSION, BUILD_VERSION, CRM_FEATURES);
     mainloop = g_main_new(FALSE);
     sysrq_init();
 
@@ -1031,7 +1041,7 @@ main(int argc, char **argv)
         }
 #if 0
         /* system() is not thread-safe, can't call from here
-         * Actually, its a pretty hacky way to try and achieve this anyway
+         * Actually, it's a pretty hacky way to try and achieve this anyway
          */
         if (system("echo 1 > /proc/sys/kernel/core_uses_pid") != 0) {
             crm_perror(LOG_ERR, "Could not enable /proc/sys/kernel/core_uses_pid");
